unit Counters;

interface

uses
  Windows, Classes, SysUtils;

type
  TBaseCounter = class(TObject)
  protected
    function GetTitle: string; virtual; abstract;
    function GetValue: extended; virtual; abstract;
    function GetByteValue: byte; virtual; abstract;
    function GetTextValue: string; virtual; abstract;
  public
    property Title     : string   read GetTitle;
    property Value     : extended read GetValue;
    property ByteValue : byte     read GetByteValue;
    property TextValue : string   read GetTextValue;
  end;

  TPercentCounter = class(TBaseCounter)
  protected
    function GetByteValue: byte; override;
    function GetTextValue: string; override;
  end;

  TNoneCounter = class(TBaseCounter)
  protected
    function GetTitle: string; override;
    function GetValue: extended; override;
    function GetByteValue: byte; override;
    function GetTextValue: string; override;    
  end;

procedure RegisterCounter(Cntr: TBaseCounter);
procedure GetCountersList(StrList: TStrings);
function  GetCounterById(id: integer): TBaseCounter;

{ =========================================================================== }

implementation

{ =========================================================================== }

var
  CountersList: TStringList;

procedure RegisterCounter(Cntr: TBaseCounter);
begin
  CountersList.AddObject(Cntr.Title, Cntr);
end;

procedure GetCountersList(StrList: TStrings);
begin
  if Assigned(StrList) then
    StrList.Assign(CountersList);
end;

function GetCounterById(id: integer): TBaseCounter;
begin
  Result := nil;
  if (id < 0) or (id >= CountersList.Count) then Exit;
  Result := TBaseCounter(CountersList.Objects[id]);
end;

{ --------------------------------------------------------------------------- }

function TPercentCounter.GetByteValue: byte;
begin
  Result := LoByte(LoWord(Round(GetValue * 2.55)));
end;

function TPercentCounter.GetTextValue: string;
begin
  Result := Format('%0.2f %%', [GetValue]);
end;

{ --------------------------------------------------------------------------- }

function TNoneCounter.GetTitle: string;
begin
  Result := ' ';
end;

function TNoneCounter.GetValue: extended;
begin
  Result := 0;
end;

function TNoneCounter.GetByteValue: byte;
begin
  Result := 0;
end;

function TNoneCounter.GetTextValue: string;
begin
  Result := '';
end;

{ =========================================================================== }

initialization
  CountersList := TStringList.Create;
  RegisterCounter(TNoneCounter.Create);

finalization
  while (CountersList.Count > 0) do begin
    CountersList.Objects[CountersList.Count - 1].Free;
    CountersList.Delete(CountersList.Count - 1);
  end;
  CountersList.Free;

end.
